/* $Id$

   Open - record and display opening times
   Copyright (C) 2001 Fraser McCrossan http://www.joat.ca

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
*/
#include <PalmOS.h>

#include "resource.h" /* resource IDs first */
#include "open.h"

FormType *form;

enum {
  NODATA,
  SAVE,
  DELETE
} disposal = NODATA; /* set when the form is initialised, and cleared
			on a save or delete, so that nothing happens
			when the form is closed */
static UInt16 whichrec; /* this is the n'th record in the
			   prefs.category after prefs.firstidx */
static Boolean createnew; /* means ignore the above value and create a
                             new one */
static UInt16 placecat; /* category that the record has/gets */
MemHandle place; /* handle of the record */
MemHandle placename; /* and to the text */
static openclose times; /* pointer to times structure within the
                            unlocked handle */
static Char triggerlabels[DAYSINWEEK*2][14]; /* each one holds a string
						of the form
						"HH:MM-HH:MM"; 14
						allows am/pm
						indicators in future */

/* used by other forms to signal this one to create a new record */
void SetNewRecord()
{
  createnew = true;
}

/* used by other forms to signal this one to edit an existing record */
void SetRecord(UInt16 r)
{
  createnew = false;
  whichrec = r;
}

/* return true if any times have been set, false otherwise */
Boolean TimesExist(openclose *t)
{
  UInt16 i;

  for (i = 0; i < DAYSINWEEK; ++i) {
    if (t->openat[i] != t->closeat[i]) {
      return true;
    }
  }
  return false;
}

void Str24H(Char *s, opentimetype openat, opentimetype closeat)
{
  StrPrintF(s, "%02d:%02d-%02d:%02d",
	    openat / MINUTESINHOUR, openat % MINUTESINHOUR,
	    closeat / MINUTESINHOUR, closeat % MINUTESINHOUR);
}

void Str12H(Char *s, opentimetype openat, opentimetype closeat)
{
  opentimetype modopenhr = openat / MINUTESINHOUR,
    modclosehr = closeat / MINUTESINHOUR;
  Char openam = (modopenhr < 12 ? 'a' : 'p'),
    closeam = (modclosehr < 12 ? 'a' : 'p');

  if (modopenhr < 1)
    modopenhr = 12;
  else if (modopenhr > 12)
    modopenhr -= 12;
  if (modclosehr < 1)
    modclosehr = 12;
  else if (modclosehr > 12)
    modclosehr -= 12;
  StrPrintF(s, "%d%02d%c-%d%02d%c",
	    modopenhr, openat % MINUTESINHOUR, openam,
	    modclosehr, closeat % MINUTESINHOUR, closeam);
}

void UpdateTriggerLabels()
{
  UInt16 i;

  for (i = 0; i < DAYSINWEEK; ++i) {
    if (times.openat[i] == times.closeat[i]) {
      StrCopy(triggerlabels[i], "--:--");
    }
    else {
      if (times.pastmid & (1<<i)) {
	/* if the past midnight is set, reverse the display */
	if (prefs.hour24)
	  Str24H(triggerlabels[i], times.closeat[i], times.openat[i]);
	else
	  Str12H(triggerlabels[i], times.closeat[i], times.openat[i]);
      }
      else {
	if (prefs.hour24)
	  Str24H(triggerlabels[i], times.openat[i], times.closeat[i]);
	else
	  Str12H(triggerlabels[i], times.openat[i], times.closeat[i]);
      }
    }
    CtlSetLabel(GetObjectPtr(form, PlaceSunTimeTrigger+i), triggerlabels[i]);
    if (times.exceptfrom[i] == times.exceptto[i]) {
      StrCopy(triggerlabels[i+DAYSINWEEK], "--:--");
    }
    else {
      if (prefs.hour24)
	Str24H(triggerlabels[i+DAYSINWEEK], times.exceptfrom[i],
	       times.exceptto[i]);
      else
	Str12H(triggerlabels[i+DAYSINWEEK], times.exceptfrom[i],
	       times.exceptto[i]);
    }
    CtlSetLabel(GetObjectPtr(form, PlaceSunExceptTrigger+i), triggerlabels[i+DAYSINWEEK]);
  }
}

void RedrawTriggers()
{
  UInt16 i;

  UpdateTriggerLabels();
  for (i = 0; i < DAYSINWEEK; ++i) {
    CtlDrawControl(GetObjectPtr(form, PlaceSunTimeTrigger+i));
    CtlDrawControl(GetObjectPtr(form, PlaceSunExceptTrigger+i));
  }
}

void DeAllocData()
{
}

Boolean InitFormData()
{
  Char *pnp;
  static Char curcatname[dmCategoryLength]; /* static because it's
                                               passed to the trigger
                                               control as it's label */

  if ((placename = MemHandleNew(sizeof(times) + MAXNAMELENGTH)) == 0) {
    FrmAlert(MemoryError);
    FrmGotoForm(MainForm);
    return false;
  }
  /* decide whether to fetch the current record or create dummy
     values to be saved later */
   if (createnew) {
    UInt16 i;
    /* set default values in our structures; we'll create a new
       record when we're done */
    for (i = 0; i < DAYSINWEEK; ++i) {
      /* these make it closed all day */
      times.openat[i] = 0;
      times.closeat[i] = 0;
      times.exceptfrom[i] = 0;
      times.exceptto[i] = 0;
    }
    times.pastmid = 0; /* clear all the past midnight bits */
    pnp = MemHandleLock(placename);
    pnp[0] = '\0';
    MemHandleUnlock(placename);
    placecat = prefs.category;
  }
  else {
    /* fetch the current record */
    pnp = MemHandleLock(placename);
    if (!QueryRecord(&db, whichrec, &times, pnp)) {
      FrmAlert(FailedQuery);
      MemHandleUnlock(placename);
      return false;
    }
    else {
      DmRecordInfo(db, whichrec, &placecat, NULL, NULL);
      MemHandleUnlock(placename);
    }
  }
  /* make sure the category is not "All" - if so, make it unfiled */
  placecat &= dmRecAttrCategoryMask;
  if (placecat == (dmAllCategories&dmRecAttrCategoryMask)) {
    placecat = dmUnfiledCategory;
  }
  /* set the popup */
  CategoryGetName(db, placecat, curcatname);
  CtlSetLabel(GetObjectPtr(form, PlaceCategoryTrigger), curcatname);
  /* assign the name handle to the field */
  FldSetTextHandle(GetObjectPtr(form, PlaceName), placename);
  /* set the 24 hour checkbox */
  CtlSetValue(GetObjectPtr(form, Place24Hr), prefs.hour24 ? 1 : 0);
  UpdateTriggerLabels();
  disposal = SAVE;
  return true;
}


Boolean SaveFormData()
{
  Char *pnp;

  /* save the current record; either create a new one, save the
     current one, or discard */
  if (disposal != NODATA) {
    if (disposal == SAVE) {
      pnp = MemHandleLock(placename);
      if (TimesExist(&times) || pnp[0] != '\0') {
	/* only save something if times or a name exists */
	if (createnew) {
	  /* first, get a pointer to the field data */
	  InsertRecord(&db, &times, pnp, placecat);
	}
	else {
	  UpdateRecord(&db, whichrec, &times, pnp, placecat);
	}
      }
      MemHandleUnlock(placename);
    }
    else if (disposal == DELETE) {
      if (!createnew) { /* if creating a new record, deletion just means
			   don't bother saving anything, only actually
			   delete if it's an old record */
	if (DmRemoveRecord(db, whichrec) != errNone) {
	  FrmAlert(FailedDelete);
	}
      }
    }
    FldSetTextHandle(GetObjectPtr(form, PlaceName), NULL);
    MemHandleFree(placename);
    placename = 0;
    disposal = NODATA;
  }
  return true;
}

Boolean PlaceFormHandler(EventType *e)
{
  TimeType opentime, closetime;
  UInt16 i;
  static Char curcatname[dmCategoryLength]; /* static because it's
                                               passed to the trigger
                                               control as it's label */

  switch (e->eType) {

  case frmOpenEvent:
    form = FrmGetActiveForm();
    if (!InitFormData()) {
      FrmGotoForm(MainForm);
      return true;
    }
    FrmDrawForm(form);
    return true;

  case frmSaveEvent:
    SaveFormData();
    return true;

  case frmCloseEvent:
    SaveFormData();
    FrmEraseForm(form);
    FrmDeleteForm(form);
    return true;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    case PlaceCategoryTrigger:
      (void) CategorySelect(db, form,
			    PlaceCategoryTrigger, PlaceCategoryList,
			    false /* in window header */,
			    &placecat, curcatname,
			    0, EditCatStr);
      CtlSetLabel(GetObjectPtr(form, PlaceCategoryTrigger), curcatname);
      return true;

    case PlaceDone:
      /* data is saved automatically if it is still good when we exit the
	 form, so just cause that to happen */
      FrmGotoForm(MainForm);
      return true;

    case PlaceDelete:
      if (FrmAlert(ConfirmDelete) == 0) {
	if (disposal == SAVE) {
	  disposal = DELETE;
	}
	FrmGotoForm(MainForm);
      } /* else don't do it */
      return true;

    case Place24Hr:
      if (CtlGetValue(GetObjectPtr(form, Place24Hr)))
	prefs.hour24 = true;
      else
	prefs.hour24 = false;
      RedrawTriggers();
      return false;

    case PlaceSunTimeTrigger:
    case PlaceMonTimeTrigger:
    case PlaceTueTimeTrigger:
    case PlaceWedTimeTrigger:
    case PlaceThuTimeTrigger:
    case PlaceFriTimeTrigger:
    case PlaceSatTimeTrigger:
      i = e->data.ctlSelect.controlID - PlaceSunTimeTrigger;
      opentime.hours = times.openat[i] / MINUTESINHOUR;
      opentime.minutes = times.openat[i] % MINUTESINHOUR;
      closetime.hours = times.closeat[i] / MINUTESINHOUR;
      closetime.minutes = times.closeat[i] % MINUTESINHOUR;
      SelectTimeV33(&opentime, &closetime,
		    (opentime.hours == closetime.hours && opentime.minutes == closetime.minutes),
		    /* have the dialog show no time if opening hours are null */
		    "", 8 /* show from 8am */);
      if (opentime.hours == closetime.hours &&
	  opentime.minutes == closetime.minutes) {
	times.openat[i] = times.closeat[i] = 0; /* standard value if
                                                   open and close
                                                   times are the same
                                                   - as is also the
                                                   case if the user
                                                   hits No Time */
	times.pastmid &= ~(1<<i); /* clear the past midnight bit too */
      }
      else {
	times.openat[i] = opentime.hours * MINUTESINHOUR + opentime.minutes;
	times.closeat[i] = closetime.hours * MINUTESINHOUR + closetime.minutes;
      }
      RedrawTriggers();
      return true;

    case PlaceSunExceptTrigger:
    case PlaceMonExceptTrigger:
    case PlaceTueExceptTrigger:
    case PlaceWedExceptTrigger:
    case PlaceThuExceptTrigger:
    case PlaceFriExceptTrigger:
    case PlaceSatExceptTrigger:
      i = e->data.ctlSelect.controlID - PlaceSunExceptTrigger;
      /* use opentime and closetime as exception time here  - why create
	 a new variable just for this? */
      opentime.hours = times.exceptfrom[i] / MINUTESINHOUR;
      opentime.minutes = times.exceptfrom[i] % MINUTESINHOUR;
      closetime.hours = times.exceptto[i] / MINUTESINHOUR;
      closetime.minutes = times.exceptto[i] % MINUTESINHOUR;
      SelectTimeV33(&opentime, &closetime,
		    (opentime.hours == closetime.hours && opentime.minutes == closetime.minutes),
		    /* have the dialog show no time if opening hours are
		       null */
		    "", 8 /* show from 8am */);
      if (opentime.hours == closetime.hours &&
	  opentime.minutes == closetime.minutes) {
	times.exceptfrom[i] = times.exceptto[i] = 0; /* standard value if
							open and close
							times are the same
							- as is also the
							case if the user
							hits No Time */
      }
      else {
	times.exceptfrom[i] = opentime.hours * MINUTESINHOUR +
	  opentime.minutes;
	times.exceptto[i] = closetime.hours * MINUTESINHOUR +
	  closetime.minutes;
	if (times.exceptfrom[i] <= times.openat[i])
	  times.exceptfrom[i] = times.openat[i]+1;
	if (times.exceptto[i] >= times.closeat[i])
	  times.exceptto[i] = times.closeat[i]-1;
      }
      RedrawTriggers();
      return true;

    case PlaceSunMidnightButton:
    case PlaceMonMidnightButton:
    case PlaceTueMidnightButton:
    case PlaceWedMidnightButton:
    case PlaceThuMidnightButton:
    case PlaceFriMidnightButton:
    case PlaceSatMidnightButton:
      i = e->data.ctlSelect.controlID - PlaceSunMidnightButton;
      /* invert the past midnight bit, but only if times are present */
      if (times.openat[i] != times.closeat[i]) {
	times.pastmid ^= (1<<i);
	RedrawTriggers();
      }
      return true;

    default:
      return false;
    }
    return false;

  case ctlExitEvent:
    /* this is used to drag a time from one control to another */

    switch (e->data.ctlExit.controlID) {

    case PlaceSunTimeTrigger:
    case PlaceMonTimeTrigger:
    case PlaceTueTimeTrigger:
    case PlaceWedTimeTrigger:
    case PlaceThuTimeTrigger:
    case PlaceFriTimeTrigger:
    case PlaceSatTimeTrigger:
      /* check if the exit event co-ords are within the bounds of any
         of the other triggers - note there is no need to worry about
         getting the same control as the start control, because a
         ctlExitEvent cannot be generated in this case */
      for (i = PlaceSunTimeTrigger; i <= PlaceSatTimeTrigger; ++i) {
	RectangleType bounds;
	FrmGetObjectBounds(form, FrmGetObjectIndex(form, i), &bounds);
	if (e->screenX >= bounds.topLeft.x && e->screenX < (bounds.topLeft.x + bounds.extent.x) &&
	    e->screenY >= bounds.topLeft.y && e->screenY < (bounds.topLeft.y + bounds.extent.y)) {
	  times.openat[i-PlaceSunTimeTrigger] = times.openat[e->data.ctlExit.controlID-PlaceSunTimeTrigger];
	  times.closeat[i-PlaceSunTimeTrigger] = times.closeat[e->data.ctlExit.controlID-PlaceSunTimeTrigger];
	  times.exceptfrom[i-PlaceSunTimeTrigger] = times.exceptfrom[e->data.ctlExit.controlID-PlaceSunTimeTrigger];
	  times.exceptto[i-PlaceSunTimeTrigger] = times.exceptto[e->data.ctlExit.controlID-PlaceSunTimeTrigger];
	  if (times.pastmid & (1<<(e->data.ctlExit.controlID-PlaceSunTimeTrigger))) {
	    times.pastmid |= 1<<(i-PlaceSunTimeTrigger); /* set the bit */
	  }
	  else {
	    times.pastmid &= ~(1<<(i-PlaceSunTimeTrigger)); /* clear the bit */
	  }
	  RedrawTriggers();
	  break;
	}
      }

    default:
      return false;
    }
    return false;

  case menuEvent:
    switch (e->data.menu.itemID) {

    case PlaceHelp:
      FrmHelp(PlaceHelpMenu);
      return true;

    case PlaceCut:
      FldCut(FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaceName)));
      return true;

    case PlaceCopy:
      FldCopy(FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaceName)));
      return true;

    case PlacePaste:
      FldPaste(FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaceName)));
      return true;

    case PlaceLookup:
      PhoneNumberLookup(FrmGetObjectPtr(form,
					FrmGetObjectIndex(form, PlaceName)));
      return true;

    default:
      return false;
    }
    return false;

  default:
    return false;
  }

  return false;
}
